# test cases

from zenroom import zenroom

def test_basic():
    script = b"print('Hello world')"
    output = zenroom.execute(script)

    assert output.decode("utf-8") == b"Hello world".decode("utf-8")

def test_encrypt_decrypt():
    encryptKeys = b"""
    {
        "device_id": "anonymous",
        "community_id": "smartcitizens",
        "community_pubkey": "BBLewg4VqLR38b38daE7Fj\/uhr543uGrEpyoPFgmFZK6EZ9g2XdK\/i65RrSJ6sJ96aXD3DJHY3Me2GJQO9\/ifjE="
    }
    """

    data = b"secret message"

    encryptScript = b"""
    curve = 'ed25519'
    keys_schema = SCHEMA.Record {
        device_id        = SCHEMA.String,
        community_id     = SCHEMA.String,
        community_pubkey = SCHEMA.String
    }

    payload_schema = SCHEMA.Record {
        device_id = SCHEMA.String,
        data      = SCHEMA.String
    }

    output_schema = SCHEMA.Record {
        device_pubkey = SCHEMA.String,
        community_id  = SCHEMA.String,
        payload       = SCHEMA.String
    }

    keys = read_json(KEYS, keys_schema)

    devkey = ECDH.keygen(curve)

    payload = {}
    payload['device_id'] = keys['device_id']
    payload['data']      = DATA
    validate(payload, payload_schema)

    header = {}
    header['device_pubkey'] = devkey:public():base64()
    header['community_id'] = keys['community_id']

    output = ECDH.encrypt(
        devkey,
        base64(keys.community_pubkey),
        MSG.pack(payload),
        MSG.pack(header)
    )

    output = map(output, O.to_base64)
    output.zenroom = VERSION
    output.encoding = 'base64'
    output.curve = curve

    print(JSON.encode(output))
    """

    decryptKeys = b"""
    {
        "community_seckey": "D19GsDTGjLBX23J281SNpXWUdu+oL6hdAJ0Zh6IrRHA="
    }
    """

    decryptScript = b"""
    keys_schema = SCHEMA.Record { community_seckey = SCHEMA.String }

    data_schema = SCHEMA.Record {
        text     = SCHEMA.string,
        iv       = SCHEMA.string,
        header   = SCHEMA.string,
        checksum = SCHEMA.string
    }

    payload_schema = SCHEMA.Record {
        device_id   = SCHEMA.String,
        data        = SCHEMA.String
    }

    data = read_json(DATA) -- TODO: data_schema validation
    keys = read_json(KEYS, keys_schema)
    head = OCTET.msgunpack( base64(data.header) )

    dashkey = ECDH.new()
    dashkey:private( base64(keys.community_seckey) )

    payload,ck = ECDH.decrypt(dashkey,
        base64( head.device_pubkey ),
        map(data, base64))

    validate(payload, payload_schema)

    print(JSON.encode(OCTET.msgunpack(payload.text)))
    """

    encryptedMessage = zenroom.execute(encryptScript, keys=encryptKeys, data=data)

    assert len(encryptedMessage) != 0

    decryptedMessage = zenroom.execute(decryptScript, keys=decryptKeys, data=encryptedMessage)

    assert decryptedMessage.encode("utf-8") == "secret message"



import ctypes
import os.path

zenroom_path = os.path.join(os.path.dirname(__file__), "_zenroom.so")

_zenroom = ctypes.CDLL(zenroom_path)

# Module variable - used to set the max size of the buffer when handling
# zenroom output
__MAX_STRING__ = 4096


class Error(Exception):
    """Base class for Zenroom errors."""

    pass


def execute(script, keys=None, data=None, conf=None, verbosity=1):
    """Invoke Zenroom, capturing and returning the output as a byte string

    This function is the primary method we expose from this wrapper library,
    which attempts to make Zenroom slightly simpler to call from Python. This
    wrapper has only been developed for a specific pilot project within DECODE,
    so beware - the code within this wrapper may be doing very bad things that
    the underlying Zenroom tool does not require.

    Args:
        script (str): Required byte string containing script which Zenroom will execute
        keys (str): Optional byte string containing keys which Zenroom will use
        data (str): Optional byte string containing data upon which Zenroom will operate
        conf (str): Optional byte string containing conf data for Zenroom
        verbosity (int): Optional int which controls Zenroom's log verbosity ranging from 1 (least verbose) up to 3 (most verbose)

    Returns:
            bytes: The output from Zenroom expressed as a byte string
    """
    stdout_buf = ctypes.create_string_buffer(b"\000", __MAX_STRING__)
    stdout_len = ctypes.c_size_t(__MAX_STRING__)
    stderr_buf = ctypes.create_string_buffer(b"\000", __MAX_STRING__)
    stderr_len = ctypes.c_size_t(__MAX_STRING__)

    err = _zenroom.zenroom_exec_tobuf(
        script,
        conf,
        keys,
        data,
        verbosity,
        ctypes.byref(stdout_buf),
        stdout_len,
        ctypes.byref(stderr_buf),
        stderr_len,
    )

    if err != 0:
        raise Error(stderr_buf.value)

    return stdout_buf.value



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zenroom",
    version="0.0.7",
    author="Sam Mulube",
    author_email="sam@thingful.net",
    description="Python wrapper for the Zenroom virtual machine",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/thingful/zenroom-py",
    packages=setuptools.find_packages(),
    package_data={"zenroom": ["_zenroom.so"]},
    setup_requires=["pytest-runner"],
    tests_require=["pytest"]
)
